Ext.namespace('dSA', 'dSA.actions', 'dSA.util', 'dSA.dialogs',
              'dSA.actions.identifydevice', 'dSS');

dSA.actions.identifydevice.Window = Ext.extend(Ext.Window,
{
    constructor: function(config)
    {
        this.devstore = config.devstore;
        this.dsmstore = config.dsmstore;

        if (config.width === undefined)
        {
            config.width = 413;
        }

        this.maskElement = config.maskElement;

        this.closeButton =
        {
            xtype: 'button',
            text: dSA._("Close"),
            scope: this,
            minWidth: 85,
            handler: function()
            {
                this.close();

                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            }
        };

        var hideSave = true;
        if (dSS.module === true)
        {
            hideSave = false;
        }

        this.saveButton = new Ext.Button(
        {
            text: dSA._("Save"),
            disabled: true,
            minWidth: 85,
            hidden: hideSave,
            scope: this,
            handler: this.saveName.createDelegate(this)
        });

        this.oldName = "";
        this.dsuid = "";
        if (dSS.module === true)
        {
            this.deviceName = new Ext.form.TextField(
            {
                fieldLabel: dSA._("Name"),
                disabled: true,
                width: 181,
                scope: this,
                enableKeyEvents: true,
                listeners:
                {
                    keyup: this.handleKeypress.createDelegate(this),
                    specialkey: this.handleSpecialKeypress.createDelegate(this)
                }
            });
        }
        else
        {
            this.deviceName = new Ext.form.Label(
            {
                fieldLabel: dSA._("Name")
            });
        }

        Ext.apply(this,
        {
            title: dSA._("Identify device"),
            bodyBorder: false,
            resizable: false,
            iconCls: 'dSA_IdentifyDeviceTitle',
            items:
            [
                {
                    xtype: 'panel',
                    padding: 9,
                    border: false,
                    bodyBorder: false,
                    frame: false,
                    layout: 'form',
                    minHeight: 175,
                    ref: 'main',
                    items:
                    [
                        {
                            xtype: 'label',
                            html: dSA._("Please press a pushbutton to show " +
                                        "device properties.")
                        },
                        {
                            xtype: 'box',
                            height: 20
                        },
                        {
                            xtype: 'label',
                            ref: 'dsm',
                            fieldLabel: dSA._("dSM")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'label',
                            ref: 'displayid',
                            fieldLabel: dSA._("Device ID")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'label',
                            ref: 'devdsid',
                            fieldLabel: dSA._("dSID")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'label',
                            ref: 'devdsuid',
                            fieldLabel: dSA._("dSUID")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'label',
                            ref: 'hwinfo',
                            fieldLabel: dSA._("HW Info")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'label',
                            ref: 'buttonmode',
                            fieldLabel: dSA._("Button input")
                        },
                        {
                            xtype: 'box',
                            height: 5
                        },
                        {
                            xtype: 'panel',
                            layout: 'absolute',
                            height: 25,
                            width: 400,
                            bodyBorder: false,
                            items:
                            [
                                {
                                    xtype: 'panel',
                                    layout: 'form',
                                    bodyBorder: false,
                                    x: 0,
                                    y: 0,
                                    items:
                                    [
                                        this.deviceName
                                    ]
                                },
                                {
                                    xtype: 'panel',
                                    x: 296,
                                    y: 0,
                                    bodyBorder: false,
                                    width: 90,
                                    height: 25,
                                    items:
                                    [
                                        this.saveButton
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            buttonAlign: 'right',
            buttons:
            [
                this.closeButton
            ]
        });

       dSA.actions.identifydevice.Window.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('afterrender',
            function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.mask();
                }
                this.el.mask(_("Activating device monitor") + '...',
                             'x-mask-loading');
                dSA.functions.StartDeviceMonitor(
                    this.deviceMonitorCallback.createDelegate(this));
            },
            this);

        this.on('beforedestroy',
            function()
            {
                this.closed = true;

                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            },
            this);
    },
    deviceMonitorCallback: function(success, task)
    {
        if (this.closed === true)
        {
            return;
        }

        this.el.unmask();
        if (success !== true)
        {
            this.close();
            return;
        }
        this.setIconClass('loading-indicator-small');
        this.taskId = task;
        dSA.functions.GetDeviceMonitorStatus(task,
                                    this.checkResults.createDelegate(this));
    },
    updateResults: function(success, data)
    {
        if (this.closed === true)
        {
            return;
        }

        this.setIconClass('dSA_IdentifyDeviceTitle');
        if (success === true)
        {
            var device = data.device;

            var rec = this.devstore.getById(device.dsuid);
            var name = "";
            if (rec !== undefined)
            {
                name = rec.get('name');
            }
            var dsmname = "";
            rec = undefined;
            var i = this.dsmstore.find('dsuid', device.dsmeter);
            if (i > -1)
            {
                rec = this.dsmstore.getAt(i);
            }
            if (rec !== undefined)
            {
                dsmname = rec.get('name');
                if ((dsmname !== undefined) && (dsmname.length === 0))
                {
                    dsmname = rec.get('displayId');
                }
            }
            else
            {
                dsmname = dSA.util.shortDSUID(device.dsmeter);
            }

            var bm = dSA.util.PushButtonModeToString(device.buttonInputMode,
                                                     this.lastKeyType,
                                                     device.buttonInputIndex,
                                                     device.buttonInputCount);

            if (device.displayId !== undefined)
            {
                this.main.displayid.setText(device.displayId);
            }
            else
            {
                this.main.displayid.setText("");
            }

            if (device.dsid !== undefined)
            {
                this.main.devdsid.setText(device.dsid);
            }
            else
            {
                this.main.devdsid.setText("");
            }
            this.main.devdsuid.setText(device.dsuid);
            this.lastDevice = device.dsuid;
            this.main.dsm.setText(dsmname);
            this.main.hwinfo.setText(device.hardwareInfo);

            this.main.buttonmode.setText(bm);

            this.oldName = name;
            if (dSS.module === true)
            {
                this.deviceName.setValue(name);
                this.deviceName.enable();
            }
            else
            {
                this.deviceName.setText(name);
            }
        }
        else
        {
            this.oldName = "";
            if (dSS.module === true)
            {
                this.deviceName.setValue("");
                this.deviceName.disable();
                this.saveButton.disable();
            }
            else
            {
                this.deviceName.setText("");
            }
        }

        var dtask = new Ext.util.DelayedTask(
                this.deviceMonitorCallback.createDelegate(this), this,
                [ true, this.taskId ]);
        dtask.delay(4000);
    },
    checkResults: function(success, data)
    {
        if (this.closed === true)
        {
            return;
        }

        if (success !== true)
        {
            this.close();
            return;
        }

        if ((parseInt('0x' + data.device) !== 0) &&
            (parseInt('0x' + data.dsmeter) !== 0))
        {
            this.dsuid = data.device;

            if ((data.device !== this.lastDevice) ||
                (data.keyType !== this.lastKeyType))
            {
                this.lastKeyType = data.keyType;
                dSA.functions.GetDevice(data.device, data.dsmeter,
                                    this.updateResults.createDelegate(this));
                return;
            }
        }
        else
        {
            this.dsuid = "";
            this.oldName = "";
            if (dSS.module === true)
            {
                this.deviceName.setValue("");
                this.deviceName.disable();
                this.saveButton.disable();
            }
            else
            {
                this.deviceName.setText("");
            }
        }

        this.setIconClass('dSA_IdentifyDeviceTitle');
        var dtask = new Ext.util.DelayedTask(
                this.deviceMonitorCallback.createDelegate(this), this,
                [ true, this.taskId ]);
        dtask.delay(4000);
    },
    handleKeypress: function(tf, e)
    {
        if (this.deviceName.getValue() !== this.oldName)
        {
            this.saveButton.enable();
        }
        else
        {
            this.saveButton.disable();
        }
    },
    handleSpecialKeypress: function(tf, e)
    {
        if (e.getKey() == e.ENTER)
        {
            this.saveButton.disable();
            if (this.deviceName.getValue() !== this.oldName)
            {
                this.saveName();
            }
        }
    },
    saveName: function()
    {
        this.saveButton.disable();
        if (dSS.module === true)
        {
            var name = this.deviceName.getValue();
            dSS.MessageBus.fireEvent('setdevicename', this.dsuid, name);
        }
        this.oldName = name;
    }
});

dSA.actions.identifydevice.IdentifyDevice = function(devstore, dsmstore, maskEl)
{
    if (devstore === undefined)
    {
        Ext.Msg.alert(dSA._("Error"), dSA._("Missing device store"));
        return;
    }

    if (dsmstore === undefined)
    {
        Ext.Msg.alert(dSA._("Error"), dSA._("Missing dSM store"));
        return;
    }

    var i = new dSA.actions.identifydevice.Window(
    {
        devstore: devstore,
        dsmstore: dsmstore,
        maskElement: maskEl
    });
    i.show();
};
